pub struct ArcConditionalBiConsumer<T, U> { /* private fields */ }Expand description
ArcConditionalBiConsumer struct
A conditional bi-consumer that wraps an ArcBiConsumer and only executes
when a predicate is satisfied. Based on Arc for thread-safe shared ownership.
Features
- Shared Ownership: Cloneable through
Arc, allows multiple owners - Thread Safe: Implements
Send + Sync, can be safely used concurrently - Conditional Execution: Only consumes when predicate returns
true - Implements BiConsumer: Can be used anywhere a
BiConsumeris expected - Readonly: Neither modifies itself nor input values
Author
Haixing Hu
Implementations
Sourceimpl<T, U> ArcConditionalBiConsumer<T, U>
impl<T, U> ArcConditionalBiConsumer<T, U>
Sourcepub fn and_then<C>(&self, next: C) -> ArcBiConsumer<T, U>
pub fn and_then<C>(&self, next: C) -> ArcBiConsumer<T, U>
Chains another bi-consumer in sequence
Combines the current conditional bi-consumer with another bi-consumer into a new bi-consumer that implements the following semantics:
When the returned bi-consumer is called with two arguments:
- First, it checks the predicate of this conditional bi-consumer
- If the predicate is satisfied, it executes the internal bi-consumer of this conditional bi-consumer
- Then, regardless of whether the predicate was
satisfied, it unconditionally executes the
nextbi-consumer
In other words, this creates a bi-consumer that conditionally executes the first action (based on the predicate), and then always executes the second action.
Parameters
next- The next bi-consumer to execute (always executed)
Returns
Returns a new combined bi-consumer
Examples
use std::sync::atomic::{AtomicI32, Ordering};
use std::sync::Arc;
use qubit_function::{BiConsumer, ArcBiConsumer};
let result = Arc::new(AtomicI32::new(0));
let result1 = std::sync::Arc::clone(&result);
let consumer = ArcBiConsumer::new(move |x: &i32, y: &i32| {
result1.fetch_add(*x + *y, Ordering::SeqCst);
});
let consumer2 = consumer.when(|x: &i32, y: &i32| *x > 0 && *y > 0);
let result2 = std::sync::Arc::clone(&result);
let chained = consumer2.and_then(ArcBiConsumer::new(move |x: &i32, y: &i32| {
result2.fetch_add(2 * (*x + *y), Ordering::SeqCst);
}));
chained.accept(&5, &3); // result = (5+3) + 2*(5+3) = 24
result.store(0, Ordering::SeqCst); // reset
chained.accept(&-5, &3); // result = 0 + 2*(-5+3) = -4 (not -8!)
assert_eq!(result.load(Ordering::SeqCst), -4);Sourcepub fn or_else<C>(&self, else_consumer: C) -> ArcBiConsumer<T, U>
pub fn or_else<C>(&self, else_consumer: C) -> ArcBiConsumer<T, U>