pub struct RcSupplier<T> { /* private fields */ }Expand description
Single-threaded shared ownership read-only supplier.
Uses Rc<dyn Fn() -> T> for single-threaded shared ownership.
Can be cloned but not sent across threads.
Ownership Model
Like ArcSupplier, methods borrow &self instead of
consuming self:
use qubit_function::{RcSupplier, Supplier};
let source = RcSupplier::new(|| 10);
let mapped = source.map(|x| x * 2);
// source is still usable here!Examples
Shared Factory
use qubit_function::{RcSupplier, Supplier};
let factory = RcSupplier::new(|| {
String::from("Hello")
});
let f1 = factory.clone();
let f2 = factory.clone();
assert_eq!(f1.get(), "Hello");
assert_eq!(f2.get(), "Hello");Reusable Transformations
use qubit_function::{RcSupplier, Supplier};
let base = RcSupplier::new(|| 10);
let doubled = base.map(|x| x * 2);
let tripled = base.map(|x| x * 3);
assert_eq!(base.get(), 10);
assert_eq!(doubled.get(), 20);
assert_eq!(tripled.get(), 30);Author
Haixing Hu
Implementations
Sourceimpl<T> RcSupplier<T>
impl<T> RcSupplier<T>
Sourcepub fn new<F>(f: F) -> Selfwhere
F: Fn() -> T + 'static,
pub fn new<F>(f: F) -> Selfwhere
F: Fn() -> T + 'static,
Creates a new supplier.
Wraps the provided closure in the appropriate smart pointer type for this supplier implementation.
Sourcepub fn new_with_name<F>(name: &str, f: F) -> Selfwhere
F: Fn() -> T + 'static,
pub fn new_with_name<F>(name: &str, f: F) -> Selfwhere
F: Fn() -> T + 'static,
Creates a new named supplier.
Wraps the provided closure and assigns it a name, which is useful for debugging and logging purposes.
Sourcepub fn new_with_optional_name<F>(f: F, name: Option<String>) -> Selfwhere
F: Fn() -> T + 'static,
pub fn new_with_optional_name<F>(f: F, name: Option<String>) -> Selfwhere
F: Fn() -> T + 'static,
Creates a new named supplier with an optional name.
Wraps the provided closure and assigns it an optional name.
Sourcepub fn map<U, M>(&self, mapper: M) -> RcSupplier<U>where
T: 'static,
M: Transformer<T, U> + 'static,
U: 'static,
pub fn map<U, M>(&self, mapper: M) -> RcSupplier<U>where
T: 'static,
M: Transformer<T, U> + 'static,
U: 'static,
Maps the output using a transformation function.
Parameters
mapper- The transformation function to apply
Returns
A new $struct_name<U> with the mapped output
Examples
use qubit_function::{$struct_name, $supplier_trait};
let source = $struct_name::new(|| 10);
let mapped = source.map(|x| x * 2);
// source is still usable
assert_eq!(mapped.get(), 20);Sourcepub fn filter<P>(&self, predicate: P) -> RcSupplier<Option<T>>where
T: 'static,
P: Predicate<T> + 'static,
pub fn filter<P>(&self, predicate: P) -> RcSupplier<Option<T>>where
T: 'static,
P: Predicate<T> + 'static,
Filters output based on a predicate.
Parameters
predicate- The predicate to test the supplied value
Returns
A new filtered $struct_name<Option<$t>>
Examples
use qubit_function::{$struct_name, $supplier_trait};
let source = $struct_name::new(|| 42);
let filtered = source.filter(|x| x % 2 == 0);
assert_eq!(filtered.get(), Some(42));Sourcepub fn zip<U, S>(&self, other: S) -> RcSupplier<(T, U)>where
T: 'static,
S: Supplier<U> + 'static,
U: 'static,
pub fn zip<U, S>(&self, other: S) -> RcSupplier<(T, U)>where
T: 'static,
S: Supplier<U> + 'static,
U: 'static,
Combines this supplier with another, producing a tuple.
Parameters
other- The other supplier to combine with
Returns
A new $struct_name<($t, U)>
Examples
use qubit_function::{$struct_name, $supplier_trait};
let first = $struct_name::new(|| 42);
let second = $struct_name::new(|| "hello");
let zipped = first.zip(second);
assert_eq!(zipped.get(), (42, "hello"));Trait Implementations
Sourceimpl<T> Clone for RcSupplier<T>
impl<T> Clone for RcSupplier<T>
Sourceimpl<T> Debug for RcSupplier<T>
impl<T> Debug for RcSupplier<T>
Sourceimpl<T> Display for RcSupplier<T>
impl<T> Display for RcSupplier<T>
Sourceimpl<T> Supplier<T> for RcSupplier<T>
impl<T> Supplier<T> for RcSupplier<T>
Sourcefn into_box(self) -> BoxSupplier<T>where
Self: 'static,
fn into_box(self) -> BoxSupplier<T>where
Self: 'static,
BoxSupplier. Read moreSourcefn into_rc(self) -> RcSupplier<T>
fn into_rc(self) -> RcSupplier<T>
RcSupplier. Read moreSourcefn to_box(&self) -> BoxSupplier<T>where
Self: 'static,
fn to_box(&self) -> BoxSupplier<T>where
Self: 'static,
BoxSupplier by cloning. Read moreSourcefn to_rc(&self) -> RcSupplier<T>
fn to_rc(&self) -> RcSupplier<T>
RcSupplier by cloning. Read moreSourcefn into_once(self) -> BoxSupplierOnce<T>where
Self: 'static,
fn into_once(self) -> BoxSupplierOnce<T>where
Self: 'static,
BoxSupplierOnce. Read moreSourcefn to_once(&self) -> BoxSupplierOnce<T>where
Self: 'static,
fn to_once(&self) -> BoxSupplierOnce<T>where
Self: 'static,
BoxSupplierOnce without consuming self Read moreSourcefn into_arc(self) -> ArcSupplier<T>
fn into_arc(self) -> ArcSupplier<T>
ArcSupplier. Read more