pub struct ArcCallable<R, E> { /* private fields */ }Expand description
Thread-safe callable.
ArcCallable<R, E> stores a Arc<Mutex<dyn FnMut() -> Result<R, E> + Send>>
and can be called repeatedly across threads.
Type Parameters
R- The success value returned by the computation.E- The error value returned when the computation fails.
Implementations
Sourceimpl<R, E> ArcCallable<R, E>
impl<R, E> ArcCallable<R, E>
Sourcepub fn new<F>(function: F) -> Self
pub fn new<F>(function: F) -> Self
Creates a new callable.
Wraps the provided closure in the appropriate smart pointer type for this callable implementation.
Examples found in repository?
89fn demo_shared_callable() {
90 println!("--- Shared callable ---");
91
92 let mut call_count = 0;
93 let shared = ArcCallable::new(move || {
94 call_count += 1;
95 Ok::<usize, String>(call_count)
96 });
97
98 let mut first = shared.clone();
99 let mut second = shared.clone();
100 println!("ArcCallable first clone: {:?}", first.call());
101 println!("ArcCallable second clone: {:?}", second.call());
102}Sourcepub fn new_with_name<F>(name: &str, function: F) -> Self
pub fn new_with_name<F>(name: &str, function: F) -> Self
Creates a new named callable.
Wraps the provided closure and assigns it a name, which is useful for debugging and logging purposes.
Sourcepub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
pub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
Creates a new named callable with an optional name.
Wraps the provided closure and assigns it an optional name.
Sourcepub fn from_supplier<S>(supplier: S) -> Self
pub fn from_supplier<S>(supplier: S) -> Self
Sourcepub fn clear_name(&mut self)
pub fn clear_name(&mut self)
Clears the name of this callable.