pub struct ArcCallableWith<T, R, E> { /* private fields */ }Expand description
Thread-safe shared callable with mutable input.
ArcCallableWith<T, R, E> stores an
Arc<Mutex<dyn FnMut(&mut T) -> Result<R, E> + Send>>.
Author
Haixing Hu
Implementations
Sourceimpl<T, R, E> ArcCallableWith<T, R, E>
impl<T, R, E> ArcCallableWith<T, R, E>
Sourcepub fn new<F>(function: F) -> Self
pub fn new<F>(function: F) -> Self
Creates a new callable-with.
Wraps the provided closure in the appropriate smart pointer type for this callable-with implementation.
Sourcepub fn new_with_name<F>(name: &str, function: F) -> Self
pub fn new_with_name<F>(name: &str, function: F) -> Self
Creates a new named callable-with.
Wraps the provided closure and assigns it a name, which is useful for debugging and logging purposes.
Sourcepub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
pub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
Creates a new named callable-with with an optional name.
Wraps the provided closure and assigns it an optional name.
Sourcepub fn clear_name(&mut self)
pub fn clear_name(&mut self)
Clears the name of this callable-with.