BoxRunnable

Struct BoxRunnable 

pub struct BoxRunnable<E> { /* private fields */ }
Box-based reusable runnable.

BoxRunnable<E> stores a Box<dyn FnMut() -> Result<(), E>> and can be executed repeatedly. It is the boxed concrete implementation of Runnable.

Type Parameters

  • E - The error value returned when the action fails.

Examples

use qubit_function::{BoxRunnable, Runnable};

let mut task = BoxRunnable::new(|| Ok::<(), String>(()));
assert_eq!(task.run(), Ok(()));

Author

Haixing Hu

Implementations

Source

impl<E> BoxRunnable<E>

pub fn new<F>(function: F) -> Self
where F: FnMut() -> Result<(), E> + 'static,

Creates a new runnable.

Wraps the provided closure in the appropriate smart pointer type for this runnable implementation.

pub fn new_with_name<F>(name: &str, function: F) -> Self
where F: FnMut() -> Result<(), E> + 'static,

Creates a new named runnable.

Wraps the provided closure and assigns it a name, which is useful for debugging and logging purposes.

pub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
where F: FnMut() -> Result<(), E> + 'static,

Creates a new named runnable with an optional name.

Wraps the provided closure and assigns it an optional name.

pub fn from_supplier<S>(supplier: S) -> Self
where S: Supplier<Result<(), E>> + 'static,

Creates a boxed runnable from a reusable supplier.

This is an explicit bridge from Supplier<Result<(), E>> to Runnable<E>.

Parameters
  • supplier - The supplier that produces the runnable result.
Returns

A new BoxRunnable<E>.

pub fn name(&self) -> Option<&str>

Gets the name of this runnable.

Returns

Returns Some(&str) if a name was set, None otherwise.

pub fn set_name(&mut self, name: &str)

Sets the name of this runnable.

Parameters
  • name - The name to set for this runnable
pub fn clear_name(&mut self)

Clears the name of this runnable.

pub fn and_then<N>(self, next: N) -> BoxRunnable<E>
where N: Runnable<E> + 'static, E: 'static,

Chains another runnable after this runnable succeeds.

The second runnable is not executed if this runnable returns Err.

Parameters
  • next - The runnable to execute after this runnable succeeds.
Returns

A runnable executing both actions in sequence.

pub fn then_callable<R, C>(self, callable: C) -> BoxCallable<R, E>
where C: Callable<R, E> + 'static, R: 'static, E: 'static,

Runs this runnable before a callable.

The callable is not executed if this runnable returns Err.

Parameters
  • callable - The callable to execute after this runnable succeeds.
Returns

A callable producing the second computation’s result.