ArcRunnableWith

Struct ArcRunnableWith 

pub struct ArcRunnableWith<T, E> { /* private fields */ }
Thread-safe shared runnable with mutable input.

ArcRunnableWith<T, E> stores an Arc<Mutex<dyn FnMut(&mut T) -> Result<(), E> + Send>>.

Author

Haixing Hu

Implementations

Source

impl<T, E> ArcRunnableWith<T, E>

pub fn new<F>(function: F) -> Self
where F: FnMut(&mut T) -> Result<(), E> + Send + 'static,

Creates a new runnable-with.

Wraps the provided closure in the appropriate smart pointer type for this runnable-with implementation.

pub fn new_with_name<F>(name: &str, function: F) -> Self
where F: FnMut(&mut T) -> Result<(), E> + Send + 'static,

Creates a new named runnable-with.

Wraps the provided closure and assigns it a name, which is useful for debugging and logging purposes.

pub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
where F: FnMut(&mut T) -> Result<(), E> + Send + 'static,

Creates a new named runnable-with with an optional name.

Wraps the provided closure and assigns it an optional name.

pub fn name(&self) -> Option<&str>

Gets the name of this runnable-with.

Returns

Returns Some(&str) if a name was set, None otherwise.

pub fn set_name(&mut self, name: &str)

Sets the name of this runnable-with.

Parameters
  • name - The name to set for this runnable-with
pub fn clear_name(&mut self)

Clears the name of this runnable-with.