ArcStatefulTester

Struct ArcStatefulTester 

pub struct ArcStatefulTester { /* private fields */ }
A thread-safe shared stateful tester backed by Arc<Mutex<_>>.

Clones of an ArcStatefulTester share the same mutable closure state and can be moved across threads.

Implementations

Source

impl ArcStatefulTester

pub fn new<F>(f: F) -> Self
where F: FnMut() -> bool + Send + 'static,

Creates a new thread-safe shared stateful tester.

pub fn and<T>(&self, next: T) -> ArcStatefulTester
where T: StatefulTester + Send + 'static,

Combines this tester with another stateful tester using logical AND.

pub fn or<T>(&self, next: T) -> ArcStatefulTester
where T: StatefulTester + Send + 'static,

Combines this tester with another stateful tester using logical OR.

pub fn nand<T>(&self, next: T) -> ArcStatefulTester
where T: StatefulTester + Send + 'static,

Combines this tester with another stateful tester using logical NAND.

pub fn xor<T>(&self, next: T) -> ArcStatefulTester
where T: StatefulTester + Send + 'static,

Combines this tester with another stateful tester using logical XOR.

pub fn nor<T>(&self, next: T) -> ArcStatefulTester
where T: StatefulTester + Send + 'static,

Combines this tester with another stateful tester using logical NOR.