pub struct RcStatefulTester { /* private fields */ }Expand description
A single-threaded shared stateful tester backed by Rc<RefCell<_>>.
Clones of an RcStatefulTester share the same mutable closure state.
Implementations
Sourceimpl RcStatefulTester
impl RcStatefulTester
Sourcepub fn and<T>(&self, next: T) -> RcStatefulTesterwhere
T: StatefulTester + 'static,
pub fn and<T>(&self, next: T) -> RcStatefulTesterwhere
T: StatefulTester + 'static,
Combines this tester with another stateful tester using logical AND.
Sourcepub fn or<T>(&self, next: T) -> RcStatefulTesterwhere
T: StatefulTester + 'static,
pub fn or<T>(&self, next: T) -> RcStatefulTesterwhere
T: StatefulTester + 'static,
Combines this tester with another stateful tester using logical OR.
Sourcepub fn nand<T>(&self, next: T) -> RcStatefulTesterwhere
T: StatefulTester + 'static,
pub fn nand<T>(&self, next: T) -> RcStatefulTesterwhere
T: StatefulTester + 'static,
Combines this tester with another stateful tester using logical NAND.
Sourcepub fn xor<T>(&self, next: T) -> RcStatefulTesterwhere
T: StatefulTester + 'static,
pub fn xor<T>(&self, next: T) -> RcStatefulTesterwhere
T: StatefulTester + 'static,
Combines this tester with another stateful tester using logical XOR.
Sourcepub fn nor<T>(&self, next: T) -> RcStatefulTesterwhere
T: StatefulTester + 'static,
pub fn nor<T>(&self, next: T) -> RcStatefulTesterwhere
T: StatefulTester + 'static,
Combines this tester with another stateful tester using logical NOR.