Transformer Types
Provides Rust implementations of transformer traits for type conversion
and value transformation. Transformers consume input values (taking
ownership) and produce output values. This is analogous to
Fn(T) -> R in Rust’s standard library.
This module provides the Transformer<T, R> trait and three
implementations:
BoxTransformer: Single ownership, not cloneableArcTransformer: Thread-safe shared ownership, cloneableRcTransformer: Single-threaded shared ownership, cloneable
Structs
- ArcConditional
Transformer - ArcConditionalTransformer struct
- ArcTransformer
- ArcTransformer - thread-safe transformer wrapper
- BoxConditional
Transformer - BoxConditionalTransformer struct
- BoxTransformer
- BoxTransformer - transformer wrapper based on
Box<dyn Fn> - RcConditional
Transformer - RcConditionalTransformer struct
- RcTransformer
- RcTransformer - single-threaded transformer wrapper
Traits
- FnTransformer
Ops - Extension trait for closures implementing the base transformer trait
- Transformer
- Transformer trait - transforms values from type T to type R
- Unary
Operator - UnaryOperator trait - marker trait for unary operators
Type Aliases
- ArcUnary
Operator - Type alias for
ArcTransformer<T, T> - BoxUnary
Operator - Type alias for
BoxTransformer<T, T> - RcUnary
Operator - Type alias for
RcTransformer<T, T>