pub struct BoxCallableWith<T, R, E> { /* private fields */ }Expand description
Box-based callable with mutable input.
BoxCallableWith<T, R, E> stores a
Box<dyn FnMut(&mut T) -> Result<R, E>> and can be called repeatedly.
Author
Haixing Hu
Implementations
Sourceimpl<T, R, E> BoxCallableWith<T, R, E>
impl<T, R, E> BoxCallableWith<T, R, E>
Sourcepub fn new<F>(function: F) -> Self
pub fn new<F>(function: F) -> Self
Creates a new callable-with.
Wraps the provided closure in the appropriate smart pointer type for this callable-with implementation.
Sourcepub fn new_with_name<F>(name: &str, function: F) -> Self
pub fn new_with_name<F>(name: &str, function: F) -> Self
Creates a new named callable-with.
Wraps the provided closure and assigns it a name, which is useful for debugging and logging purposes.
Sourcepub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
pub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
Creates a new named callable-with with an optional name.
Wraps the provided closure and assigns it an optional name.
Sourcepub fn clear_name(&mut self)
pub fn clear_name(&mut self)
Clears the name of this callable-with.
Sourcepub fn map<U, M>(self, mapper: M) -> BoxCallableWith<T, U, E>where
M: FnMut(R) -> U + 'static,
T: 'static,
R: 'static,
E: 'static,
pub fn map<U, M>(self, mapper: M) -> BoxCallableWith<T, U, E>where
M: FnMut(R) -> U + 'static,
T: 'static,
R: 'static,
E: 'static,
Sourcepub fn map_err<E2, M>(self, mapper: M) -> BoxCallableWith<T, R, E2>where
M: FnMut(E) -> E2 + 'static,
T: 'static,
R: 'static,
E: 'static,
pub fn map_err<E2, M>(self, mapper: M) -> BoxCallableWith<T, R, E2>where
M: FnMut(E) -> E2 + 'static,
T: 'static,
R: 'static,
E: 'static,
Sourcepub fn and_then<U, N>(self, next: N) -> BoxCallableWith<T, U, E>
pub fn and_then<U, N>(self, next: N) -> BoxCallableWith<T, U, E>