pub struct BoxRunnable<E> { /* private fields */ }Expand description
Box-based reusable runnable.
BoxRunnable<E> stores a Box<dyn FnMut() -> Result<(), E>> and can be
executed repeatedly. It is the boxed concrete implementation of
Runnable.
Type Parameters
E- The error value returned when the action fails.
Examples
use qubit_function::{BoxRunnable, Runnable};
let mut task = BoxRunnable::new(|| Ok::<(), String>(()));
assert_eq!(task.run(), Ok(()));Author
Haixing Hu
Implementations
Sourceimpl<E> BoxRunnable<E>
impl<E> BoxRunnable<E>
Sourcepub fn new<F>(function: F) -> Self
pub fn new<F>(function: F) -> Self
Creates a new runnable.
Wraps the provided closure in the appropriate smart pointer type for this runnable implementation.
Sourcepub fn new_with_name<F>(name: &str, function: F) -> Self
pub fn new_with_name<F>(name: &str, function: F) -> Self
Creates a new named runnable.
Wraps the provided closure and assigns it a name, which is useful for debugging and logging purposes.
Sourcepub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
pub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
Creates a new named runnable with an optional name.
Wraps the provided closure and assigns it an optional name.
Sourcepub fn from_supplier<S>(supplier: S) -> Self
pub fn from_supplier<S>(supplier: S) -> Self
Sourcepub fn clear_name(&mut self)
pub fn clear_name(&mut self)
Clears the name of this runnable.
Sourcepub fn and_then<N>(self, next: N) -> BoxRunnable<E>where
N: Runnable<E> + 'static,
E: 'static,
pub fn and_then<N>(self, next: N) -> BoxRunnable<E>where
N: Runnable<E> + 'static,
E: 'static,
Sourcepub fn then_callable<R, C>(self, callable: C) -> BoxCallable<R, E>where
C: Callable<R, E> + 'static,
R: 'static,
E: 'static,
pub fn then_callable<R, C>(self, callable: C) -> BoxCallable<R, E>where
C: Callable<R, E> + 'static,
R: 'static,
E: 'static,