pub struct ArcRunnableWith<T, E> { /* private fields */ }Expand description
Thread-safe shared runnable with mutable input.
ArcRunnableWith<T, E> stores an
Arc<Mutex<dyn FnMut(&mut T) -> Result<(), E> + Send>>.
Author
Haixing Hu
Implementations
Sourceimpl<T, E> ArcRunnableWith<T, E>
impl<T, E> ArcRunnableWith<T, E>
Sourcepub fn new<F>(function: F) -> Self
pub fn new<F>(function: F) -> Self
Creates a new runnable-with.
Wraps the provided closure in the appropriate smart pointer type for this runnable-with implementation.
Sourcepub fn new_with_name<F>(name: &str, function: F) -> Self
pub fn new_with_name<F>(name: &str, function: F) -> Self
Creates a new named runnable-with.
Wraps the provided closure and assigns it a name, which is useful for debugging and logging purposes.
Sourcepub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
pub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
Creates a new named runnable-with with an optional name.
Wraps the provided closure and assigns it an optional name.
Sourcepub fn clear_name(&mut self)
pub fn clear_name(&mut self)
Clears the name of this runnable-with.