pub struct BoxRunnableWith<T, E> { /* private fields */ }Expand description
Box-based runnable with mutable input.
BoxRunnableWith<T, E> stores a
Box<dyn FnMut(&mut T) -> Result<(), E>> and can be called repeatedly.
Author
Haixing Hu
Implementations
Sourceimpl<T, E> BoxRunnableWith<T, E>
impl<T, E> BoxRunnableWith<T, E>
Sourcepub fn new<F>(function: F) -> Self
pub fn new<F>(function: F) -> Self
Creates a new runnable-with.
Wraps the provided closure in the appropriate smart pointer type for this runnable-with implementation.
Sourcepub fn new_with_name<F>(name: &str, function: F) -> Self
pub fn new_with_name<F>(name: &str, function: F) -> Self
Creates a new named runnable-with.
Wraps the provided closure and assigns it a name, which is useful for debugging and logging purposes.
Sourcepub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
pub fn new_with_optional_name<F>(function: F, name: Option<String>) -> Self
Creates a new named runnable-with with an optional name.
Wraps the provided closure and assigns it an optional name.
Sourcepub fn clear_name(&mut self)
pub fn clear_name(&mut self)
Clears the name of this runnable-with.
Sourcepub fn and_then<N>(self, next: N) -> BoxRunnableWith<T, E>where
N: RunnableWith<T, E> + 'static,
T: 'static,
E: 'static,
pub fn and_then<N>(self, next: N) -> BoxRunnableWith<T, E>where
N: RunnableWith<T, E> + 'static,
T: 'static,
E: 'static,
Sourcepub fn then_callable_with<R, C>(self, callable: C) -> BoxCallableWith<T, R, E>where
C: CallableWith<T, R, E> + 'static,
T: 'static,
R: 'static,
E: 'static,
pub fn then_callable_with<R, C>(self, callable: C) -> BoxCallableWith<T, R, E>where
C: CallableWith<T, R, E> + 'static,
T: 'static,
R: 'static,
E: 'static,