pub struct ArcStatefulTester { /* private fields */ }Expand description
A thread-safe shared stateful tester backed by Arc<Mutex<_>>.
Clones of an ArcStatefulTester share the same mutable closure state and
can be moved across threads.
Implementations
Sourceimpl ArcStatefulTester
impl ArcStatefulTester
Sourcepub fn and<T>(&self, next: T) -> ArcStatefulTesterwhere
T: StatefulTester + Send + 'static,
pub fn and<T>(&self, next: T) -> ArcStatefulTesterwhere
T: StatefulTester + Send + 'static,
Combines this tester with another stateful tester using logical AND.
Sourcepub fn or<T>(&self, next: T) -> ArcStatefulTesterwhere
T: StatefulTester + Send + 'static,
pub fn or<T>(&self, next: T) -> ArcStatefulTesterwhere
T: StatefulTester + Send + 'static,
Combines this tester with another stateful tester using logical OR.
Sourcepub fn nand<T>(&self, next: T) -> ArcStatefulTesterwhere
T: StatefulTester + Send + 'static,
pub fn nand<T>(&self, next: T) -> ArcStatefulTesterwhere
T: StatefulTester + Send + 'static,
Combines this tester with another stateful tester using logical NAND.
Sourcepub fn xor<T>(&self, next: T) -> ArcStatefulTesterwhere
T: StatefulTester + Send + 'static,
pub fn xor<T>(&self, next: T) -> ArcStatefulTesterwhere
T: StatefulTester + Send + 'static,
Combines this tester with another stateful tester using logical XOR.
Sourcepub fn nor<T>(&self, next: T) -> ArcStatefulTesterwhere
T: StatefulTester + Send + 'static,
pub fn nor<T>(&self, next: T) -> ArcStatefulTesterwhere
T: StatefulTester + Send + 'static,
Combines this tester with another stateful tester using logical NOR.