pub struct BoxStatefulTester { /* private fields */ }Expand description
A single-ownership stateful tester backed by Box<dyn FnMut() -> bool>.
BoxStatefulTester is the owned wrapper for zero-argument tests that need
native FnMut() -> bool semantics. Calling StatefulTester::test may
mutate state captured by the wrapped closure.
Implementations
Sourceimpl BoxStatefulTester
impl BoxStatefulTester
Sourcepub fn and<T>(self, next: T) -> BoxStatefulTesterwhere
T: StatefulTester + 'static,
pub fn and<T>(self, next: T) -> BoxStatefulTesterwhere
T: StatefulTester + 'static,
Combines this tester with another stateful tester using logical AND.
The second tester is evaluated only when the first tester returns
true.
Sourcepub fn or<T>(self, next: T) -> BoxStatefulTesterwhere
T: StatefulTester + 'static,
pub fn or<T>(self, next: T) -> BoxStatefulTesterwhere
T: StatefulTester + 'static,
Combines this tester with another stateful tester using logical OR.
The second tester is evaluated only when the first tester returns
false.
Sourcepub fn nand<T>(self, next: T) -> BoxStatefulTesterwhere
T: StatefulTester + 'static,
pub fn nand<T>(self, next: T) -> BoxStatefulTesterwhere
T: StatefulTester + 'static,
Combines this tester with another stateful tester using logical NAND.
Sourcepub fn xor<T>(self, next: T) -> BoxStatefulTesterwhere
T: StatefulTester + 'static,
pub fn xor<T>(self, next: T) -> BoxStatefulTesterwhere
T: StatefulTester + 'static,
Combines this tester with another stateful tester using logical XOR.
Sourcepub fn nor<T>(self, next: T) -> BoxStatefulTesterwhere
T: StatefulTester + 'static,
pub fn nor<T>(self, next: T) -> BoxStatefulTesterwhere
T: StatefulTester + 'static,
Combines this tester with another stateful tester using logical NOR.